//+------------------------------------------------------------------+
//| Stochastic Momentum Index Developed By William Blau, 2013 Version|
//+------------------------------------------------------------------+
//---- Indicator Settings
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  SteelBlue
#property indicator_color2  Tomato
#property indicator_color3  Yellow
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_level1  0
#property indicator_levelcolor DimGray
//---- Input Parameters
extern int  QPeriod = 13;
extern int  RPeriod = 15;
extern int  SPeriod = 5 ;
extern int  Step    = 5 ;
extern bool Arrow   = False;
//---- Buffers and Variables
double Up  [];
double SMI [];
double Down[];
double Flat[];
string Prefix;

//+-------------------------------------------------------+
//| Custom Indicator Initialization Function              |
//+-------------------------------------------------------+
int init()
  {
   IndicatorDigits(Digits+1);
   IndicatorBuffers(4);
   SetIndexBuffer(0,Up  );SetIndexStyle(0,DRAW_ARROW);SetIndexArrow(0,159);SetIndexLabel(0,"SMI Up"  );
   SetIndexBuffer(1,Down);SetIndexStyle(1,DRAW_ARROW);SetIndexArrow(1,159);SetIndexLabel(1,"SMI Down");
   SetIndexBuffer(2,Flat);SetIndexStyle(2,DRAW_ARROW);SetIndexArrow(2,159);SetIndexLabel(2,"SMI Flat");
   SetIndexBuffer(3,SMI );
   Prefix = "SMI";
   IndicatorShortName(Prefix+ " "+QPeriod+","+RPeriod+","+SPeriod+", Current Value:");
//----  Initialization is done.
   return(0);
  }

//+-------------------------------------------------------+
//| Custom Indicator Deinitialization Function            |
//+-------------------------------------------------------+
int deinit()
  {
   ArrowRemover();
   return(0);
  }

//+-------------------------------------------------------+
//| Stochastic Momentum Index                             |
//+-------------------------------------------------------+
int start()
  {
   int CountedBars = IndicatorCounted();
   if (CountedBars < 0)  return(-1);
   if (CountedBars > 0)  CountedBars--;
   int Limit=Bars-QPeriod;
//---- Stochastic Momentum Index Iteration Function
   for (int i=Limit-RPeriod-SPeriod; i>=0; i--)
     {
      double HQ = iEMA(iEMA(          High[iHighest(Symbol(),Period(),MODE_HIGH,QPeriod,i)]-Low[iLowest(Symbol(),Period(),MODE_LOW,QPeriod,i)],   RPeriod,i,0),SPeriod,i,1);
      double SM = iEMA(iEMA(Close[i]-(High[iHighest(Symbol(),Period(),MODE_HIGH,QPeriod,i)]+Low[iLowest(Symbol(),Period(),MODE_LOW,QPeriod,i)])/2,RPeriod,i,2),SPeriod,i,3);
      SMI[i]=100*(SM/0.5/HQ);
      // Step Handling
      int STrend;
      double SMinP,SMaxP,SMinS,SMaxS;
      SMaxP = SMI[i]+2*Step;
      SMinP = SMI[i]-2*Step;
      if (SMI[i] > SMaxS)  STrend =  1;
      if (SMI[i] < SMinS)  STrend = -1;
      if (STrend > 0 && SMinP < SMinS)  SMinP = SMinS;
      if (STrend < 0 && SMaxP > SMaxS)  SMaxP = SMaxS;
      if (STrend > 0)  SMI[i] = SMinP+Step;
      if (STrend < 0)  SMI[i] = SMaxP-Step;
      SMinS = SMinP;
      SMaxS = SMaxP;
      // Color Handling
      Up  [i] = EMPTY_VALUE;
      Down[i] = EMPTY_VALUE;
      Flat[i] = EMPTY_VALUE;
      if      (SMI[i+1] > SMI[i])  Down[i] = SMI[i];
      else if (SMI[i+1] < SMI[i])  Up  [i] = SMI[i];
      else                         Flat[i] = SMI[i];
      // Arrow Management
      if (Arrow)
        {
         if (SMI[i] > 0 && SMI[i+1] < 0)  ArrowDrawer("Up"   ,i);
         if (SMI[i] < 0 && SMI[i+1] > 0)  ArrowDrawer("Down" ,i);
         if (SMI[i]   >    SMI[i+1]    )  ArrowDrawer("PUp"  ,i);
         if (SMI[i]   <    SMI[i+1]    )  ArrowDrawer("PDown",i);
        }
     }
   return(0);
  }

//+-------------------------------------------------------+
//| User-Defined Functions                                |
//+-------------------------------------------------------+
//----- EMA
double WorkEMA[][7];
double iEMA(double price, double period, int r, int No=0)
  {
   if (ArrayRange(WorkEMA,0)!= Bars)  ArrayResize(WorkEMA,Bars);
   r = Bars-r-1;
   double Alpha = 2.0/(1.0+period);
   WorkEMA[r][No] = WorkEMA[r-1][No]+Alpha*(price-WorkEMA[r-1][No]);
   return(WorkEMA[r][No]);
  }

//---- Arrow Remover
void ArrowRemover()
  {
   int n = ObjectsTotal();
   for (int i=n-1; i>=0; i--)
    {
     string SName = ObjectName(i);
	  if (StringFind(SName,Prefix) == 0)  ObjectDelete(SName);
    }
  }

//---- Arrow Drawer
void ArrowDrawer(string SSName, int i)
  {
   static string   PreviousAlert = "Nothing";
   static datetime PreviousTime;
   string SName = Prefix+" "+SSName+" "+TimeToStr(Time[i],TIME_DATE|TIME_MINUTES);
   ObjectDelete(SName);
   ObjectCreate(SName,OBJ_ARROW,0,Time[i],0);
   double Gap = 3.0*iATR(Symbol(),0,20,i)/3.0;
   if (PreviousAlert != SSName || PreviousTime != Time[0])
     {
      PreviousAlert = SSName;
      PreviousTime  = Time[0];
      if (SSName == "Up")
        {
         ObjectSet(SName,OBJPROP_ARROWCODE,233    );
         ObjectSet(SName,OBJPROP_PRICE1,Low[i]-Gap);
         ObjectSet(SName,OBJPROP_COLOR,RoyalBlue  );
        }
      if (SSName == "PUp")
        {
         ObjectSet(SName,OBJPROP_ARROWCODE,225    );
         ObjectSet(SName,OBJPROP_PRICE1,Low[i]-Gap);
         ObjectSet(SName,OBJPROP_COLOR,SteelBlue  );
        }
      if (SSName == "Down")
        {
         ObjectSet(SName,OBJPROP_ARROWCODE,234     );
         ObjectSet(SName,OBJPROP_PRICE1,High[i]+Gap);
         ObjectSet(SName,OBJPROP_COLOR,Red         );
        }
      if (SSName == "PDown")
        {
         ObjectSet(SName,OBJPROP_ARROWCODE,226     );
         ObjectSet(SName,OBJPROP_PRICE1,High[i]+Gap);
         ObjectSet(SName,OBJPROP_COLOR,Salmon      );
        }
     } 
   ObjectSet(SName,OBJPROP_WIDTH,1);
  }
//+-------------------------------------------------------+